module MidwayContestInteractableGondola

using ..Ahorn, Maple

@mapdef Entity "MidwayContestHelper/InteractableGondola" InteractableGondola(x::Integer, y::Integer, atEnd::Bool=false, speed::Number=2.5, Persistent::Bool=false)

const placements = Ahorn.PlacementDict(
    "Interactable Gondola (Midway Contest Helper)" => Ahorn.EntityPlacement(
        InteractableGondola,
        "line",
        Dict{String, Any}(),
        function(entity)
            entity.data["nodes"] = [
                (Int(entity.data["x"]) + 8, Int(entity.data["y"])),
                (Int(entity.data["x"]) - 128, Int(entity.data["y"])-16),
                (Int(entity.data["x"]) + 168, Int(entity.data["y"])-56)
                
            ]
        end
    )
)

frontSprite = "objects/gondola/front"
backSprite = "objects/gondola/back"
topSprite = "objects/gondola/top"
leverSprite = "objects/MWC2022/contesthelper/asset/lever/lever00"
leftSprite = "objects/gondola/cliffsideLeft"
rightSprite = "objects/gondola/cliffsideRight"

renderOffsetY = -64

Ahorn.nodeLimits(entity::InteractableGondola) = 3, 3

function Ahorn.selection(entity::InteractableGondola)
    x, y = Ahorn.position(entity)

    destX, destY = Int.(entity.data["nodes"][1])
    lev1X, lev1Y = Int.(entity.data["nodes"][2])
    lev2X, lev2Y = Int.(entity.data["nodes"][3])

    return [
        Ahorn.getSpriteRectangle(frontSprite, x, y + renderOffsetY, jx=0.5, jy=0.0),
        Ahorn.getSpriteRectangle(rightSprite, destX + 144, destY - 104, jx=0.0, jy=0.5, sx=-1),
        Ahorn.getSpriteRectangle(leverSprite, lev1X, lev1Y + 8, jx=0.5, jy=0.5),
        Ahorn.getSpriteRectangle(leverSprite, lev2X, lev2Y + 8, jx=0.5, jy=0.5)
    ]
end

function Ahorn.renderAbs(ctx::Ahorn.Cairo.CairoContext, entity::InteractableGondola, room::Maple.Room)
    x, y = Ahorn.position(entity)

    nodes = get(entity.data, "nodes", ())

    if !isempty(nodes)
        top = Ahorn.getSprite(topSprite, "Gameplay")

        destX, destY = Int.(nodes[1])
        lev1X, lev1Y = Int.(nodes[2])
        lev2X, lev2Y = Int.(nodes[3])

        wireLeftX = x - 124 + 40
        wireLeftY = y - 12

        wireRightX = destX + 144 - 40
        wireRightY = destY - 104 - 4

        topX = x
        topY = y + renderOffsetY + top.height

        Ahorn.Cairo.save(ctx)

        Ahorn.setSourceColor(ctx, (0.0, 0.0, 0.0, 1.0))
        Ahorn.set_antialias(ctx, 1)
        Ahorn.set_line_width(ctx, 1);

        for i in 0:1
            Ahorn.move_to(ctx, wireLeftX, wireLeftY + i)
            Ahorn.line_to(ctx, topX, topY + i)

            Ahorn.stroke(ctx)

            Ahorn.move_to(ctx, topX, topY + i)
            Ahorn.line_to(ctx, wireRightX, wireRightY + i)

            Ahorn.stroke(ctx)
        end

        # Don't rotate the top sprite, it's super broken in game
        Ahorn.drawSprite(ctx, frontSprite, x, y + renderOffsetY, jx=0.5, jy=0.0)
        Ahorn.drawSprite(ctx, topSprite, x, y + renderOffsetY, jx=0.5, jy=0.0)
        Ahorn.drawSprite(ctx, backSprite, x, y + renderOffsetY, jx=0.5, jy=0.0)

        Ahorn.drawSprite(ctx, leftSprite, x - 124, y, jx=0.0, jy=1.0)
        Ahorn.drawSprite(ctx, rightSprite, destX + 144, destY - 104, jx=0.0, jy=0.5, sx=-1)

        Ahorn.drawSprite(ctx, leverSprite, lev1X, lev1Y + 8)
        Ahorn.drawSprite(ctx, leverSprite, lev2X, lev2Y + 8)
        Ahorn.Cairo.restore(ctx)
    end
end

end